@@ -3,29 +3,36 @@ from datetime import datetime
 from datetime import timedelta
 from datetime import timezone
 from hmac import compare_digest
+from typing import Any
+from typing import Iterable
+from typing import List
+from typing import Type
+from typing import Union
 
 import jwt
+from flask.json import JSONEncoder
 
 from flask_jwt_extended.exceptions import CSRFError
 from flask_jwt_extended.exceptions import JWTDecodeError
+from flask_jwt_extended.typing import ExpiresDelta
 
 
 def _encode_jwt(
-    algorithm,
-    audience,
-    claim_overrides,
-    csrf,
-    expires_delta,
-    fresh,
-    header_overrides,
-    identity,
-    identity_claim_key,
-    issuer,
-    json_encoder,
-    secret,
-    token_type,
-    nbf,
-):
+    algorithm: str,
+    audience: Union[str, Iterable[str]],
+    claim_overrides: dict,
+    csrf: bool,
+    expires_delta: ExpiresDelta,
+    fresh: bool,
+    header_overrides: dict,
+    identity: Any,
+    identity_claim_key: str,
+    issuer: str,
+    json_encoder: Type[JSONEncoder],
+    secret: str,
+    token_type: str,
+    nbf: bool,
+) -> str:
     now = datetime.now(timezone.utc)
 
     if isinstance(fresh, timedelta):
@@ -61,23 +68,23 @@ def _encode_jwt(
         token_data,
         secret,
         algorithm,
-        json_encoder=json_encoder,
+        json_encoder=json_encoder,  # type: ignore
         headers=header_overrides,
     )
 
 
 def _decode_jwt(
-    algorithms,
-    allow_expired,
-    audience,
-    csrf_value,
-    encoded_token,
-    identity_claim_key,
-    issuer,
-    leeway,
-    secret,
-    verify_aud,
-):
+    algorithms: List,
+    allow_expired: bool,
+    audience: Union[str, Iterable[str]],
+    csrf_value: str,
+    encoded_token: str,
+    identity_claim_key: str,
+    issuer: str,
+    leeway: int,
+    secret: str,
+    verify_aud: bool,
+) -> dict:
     options = {"verify_aud": verify_aud}
     if allow_expired:
         options["verify_exp"] = False
