@@ -1,16 +1,20 @@
+import datetime
+from typing import Any
+from typing import Optional
+
 import jwt
 from flask import _request_ctx_stack
+from flask import Response
 from werkzeug.local import LocalProxy
 
 from flask_jwt_extended.config import config
 from flask_jwt_extended.internal_utils import get_jwt_manager
 
-
 # Proxy to access the current user
 current_user = LocalProxy(lambda: get_current_user())
 
 
-def get_jwt():
+def get_jwt() -> dict:
     """
     In a protected endpoint, this will return the python dictionary which has
     the payload of the JWT that is accessing the endpoint. If no JWT is present
@@ -28,7 +32,7 @@ def get_jwt():
     return decoded_jwt
 
 
-def get_jwt_header():
+def get_jwt_header() -> dict:
     """
     In a protected endpoint, this will return the python dictionary which has
     the header of the JWT that is accessing the endpoint. If no JWT is present
@@ -46,7 +50,7 @@ def get_jwt_header():
     return decoded_header
 
 
-def get_jwt_identity():
+def get_jwt_identity() -> Any:
     """
     In a protected endpoint, this will return the identity of the JWT that is
     accessing the endpoint. If no JWT is present due to
@@ -58,7 +62,7 @@ def get_jwt_identity():
     return get_jwt().get(config.identity_claim_key, None)
 
 
-def get_jwt_request_location():
+def get_jwt_request_location() -> Optional[str]:
     """
     In a protected endpoint, this will return the "location" at which the JWT
     that is accessing the endpoint was found--e.g., "cookies", "query-string",
@@ -66,14 +70,13 @@ def get_jwt_request_location():
     None is returned.
 
     :return:
-        The location of the JWT in the current request; e.g., cookies",
+        The location of the JWT in the current request; e.g., "cookies",
         "query-string", "headers", or "json"
     """
-    location = getattr(_request_ctx_stack.top, "jwt_location", None)
-    return location
+    return getattr(_request_ctx_stack.top, "jwt_location", None)
 
 
-def get_current_user():
+def get_current_user() -> Any:
     """
     In a protected endpoint, this will return the user object for the JWT that
     is accessing the endpoint.
@@ -97,14 +100,16 @@ def get_current_user():
     return jwt_user_dict["loaded_user"]
 
 
-def decode_token(encoded_token, csrf_value=None, allow_expired=False):
+def decode_token(
+    encoded_token: str, csrf_value: str = None, allow_expired: bool = False
+) -> dict:
     """
     Returns the decoded token (python dict) from an encoded JWT. This does all
-    the checks to insure that the decoded token is valid before returning it.
+    the checks to ensure that the decoded token is valid before returning it.
 
     This will not fire the user loader callbacks, save the token for access
     in protected endpoints, checked if a token is revoked, etc. This is puerly
-    used to insure that a JWT is valid.
+    used to ensure that a JWT is valid.
 
     :param encoded_token:
         The encoded JWT to decode.
@@ -123,9 +128,9 @@ def decode_token(encoded_token, csrf_value=None, allow_expired=False):
 
 
 def create_access_token(
-    identity,
-    fresh=False,
-    expires_delta=None,
+    identity: Any,
+    fresh: bool = False,
+    expires_delta: datetime.timedelta = None,
     additional_claims=None,
     additional_headers=None,
 ):
@@ -177,7 +182,10 @@ def create_access_token(
 
 
 def create_refresh_token(
-    identity, expires_delta=None, additional_claims=None, additional_headers=None
+    identity: Any,
+    expires_delta: datetime.timedelta = None,
+    additional_claims=None,
+    additional_headers=None,
 ):
     """
     Create a new refresh token.
@@ -219,7 +227,7 @@ def create_refresh_token(
     )
 
 
-def get_unverified_jwt_headers(encoded_token):
+def get_unverified_jwt_headers(encoded_token: str) -> dict:
     """
     Returns the Headers of an encoded JWT without verifying the signature of the JWT.
 
@@ -232,7 +240,7 @@ def get_unverified_jwt_headers(encoded_token):
     return jwt.get_unverified_header(encoded_token)
 
 
-def get_jti(encoded_token):
+def get_jti(encoded_token: str) -> Optional[str]:
     """
     Returns the JTI (unique identifier) of an encoded JWT
 
@@ -240,12 +248,12 @@ def get_jti(encoded_token):
         The encoded JWT to get the JTI from.
 
     :return:
-        The JTI (unique identifier) of a JWT.
+        The JTI (unique identifier) of a JWT, if it is present.
     """
     return decode_token(encoded_token).get("jti")
 
 
-def get_csrf_token(encoded_token):
+def get_csrf_token(encoded_token: str) -> str:
     """
     Returns the CSRF double submit token from an encoded JWT.
 
@@ -259,7 +267,9 @@ def get_csrf_token(encoded_token):
     return token["csrf"]
 
 
-def set_access_cookies(response, encoded_access_token, max_age=None, domain=None):
+def set_access_cookies(
+    response: Response, encoded_access_token: str, max_age=None, domain=None
+) -> None:
     """
     Modifiy a Flask Response to set a cookie containing the access JWT.
     Also sets the corresponding CSRF cookies if ``JWT_CSRF_IN_COOKIES`` is ``True``
@@ -307,7 +317,12 @@ def set_access_cookies(response, encoded_access_token, max_age=None, domain=None
         )
 
 
-def set_refresh_cookies(response, encoded_refresh_token, max_age=None, domain=None):
+def set_refresh_cookies(
+    response: Response,
+    encoded_refresh_token: str,
+    max_age: int = None,
+    domain: str = None,
+) -> None:
     """
     Modifiy a Flask Response to set a cookie containing the refresh JWT.
     Also sets the corresponding CSRF cookies if ``JWT_CSRF_IN_COOKIES`` is ``True``
@@ -355,7 +370,7 @@ def set_refresh_cookies(response, encoded_refresh_token, max_age=None, domain=No
         )
 
 
-def unset_jwt_cookies(response, domain=None):
+def unset_jwt_cookies(response: Response, domain: str = None) -> None:
     """
     Modifiy a Flask Response to delete the cookies containing access or refresh
     JWTs.  Also deletes the corresponding CSRF cookies if applicable.
@@ -367,7 +382,7 @@ def unset_jwt_cookies(response, domain=None):
     unset_refresh_cookies(response, domain)
 
 
-def unset_access_cookies(response, domain=None):
+def unset_access_cookies(response: Response, domain: str = None) -> None:
     """
     Modifiy a Flask Response to delete the cookie containing an access JWT.
     Also deletes the corresponding CSRF cookie if applicable.
@@ -405,7 +420,7 @@ def unset_access_cookies(response, domain=None):
         )
 
 
-def unset_refresh_cookies(response, domain=None):
+def unset_refresh_cookies(response: Response, domain: str = None) -> None:
     """
     Modifiy a Flask Response to delete the cookie containing a refresh JWT.
     Also deletes the corresponding CSRF cookie if applicable.
