@@ -2,6 +2,11 @@ from datetime import datetime
 from datetime import timezone
 from functools import wraps
 from re import split
+from typing import Any
+from typing import Optional
+from typing import Sequence
+from typing import Tuple
+from typing import Union
 
 from flask import _request_ctx_stack
 from flask import current_app
@@ -23,8 +28,10 @@ from flask_jwt_extended.internal_utils import verify_token_type
 from flask_jwt_extended.utils import decode_token
 from flask_jwt_extended.utils import get_unverified_jwt_headers
 
+LocationType = Union[str, Sequence, None]
 
-def _verify_token_is_fresh(jwt_header, jwt_data):
+
+def _verify_token_is_fresh(jwt_header: dict, jwt_data: dict) -> None:
     fresh = jwt_data["fresh"]
     if isinstance(fresh, bool):
         if not fresh:
@@ -35,7 +42,13 @@ def _verify_token_is_fresh(jwt_header, jwt_data):
             raise FreshTokenRequired("Fresh token required", jwt_header, jwt_data)
 
 
-def verify_jwt_in_request(optional=False, fresh=False, refresh=False, locations=None):
+def verify_jwt_in_request(
+    optional: bool = False,
+    fresh: bool = False,
+    refresh: bool = False,
+    locations: LocationType = None,
+    verify_type: bool = True,
+) -> Optional[Tuple[dict, dict]]:
     """
     Verify that a valid JWT is present in the request, unless ``optional=True`` in
     which case no JWT is also considered valid.
@@ -49,26 +62,34 @@ def verify_jwt_in_request(optional=False, fresh=False, refresh=False, locations=
         Defaults to ``False``.
 
     :param refresh:
-        If ``True``, require a refresh JWT to be verified.
+        If ``True``, requires a refresh JWT to access this endpoint. If ``False``,
+        requires an access JWT to access this endpoint. Defaults to ``False``
 
     :param locations:
         A location or list of locations to look for the JWT in this request, for
         example ``'headers'`` or ``['headers', 'cookies']``. Defaults to ``None``
         which indicates that JWTs will be looked for in the locations defined by the
         ``JWT_TOKEN_LOCATION`` configuration option.
+
+    :param verify_type:
+        If ``True``, the token type (access or refresh) will be checked according
+        to the ``refresh`` argument. If ``False``, type will not be checked and both
+        access and refresh tokens will be accepted.
+
+    :return:
+        A tuple containing the jwt_header and the jwt_data if a valid JWT is
+        present in the request. If ``optional=True`` and no JWT is in the request,
+        ``None`` will be returned instead. Raise an exception if an invalid JWT
+        is in the request.
     """
     if request.method in config.exempt_methods:
-        return
+        return None
 
     try:
-        if refresh:
-            jwt_data, jwt_header, jwt_location = _decode_jwt_from_request(
-                locations, fresh, refresh=True
-            )
-        else:
-            jwt_data, jwt_header, jwt_location = _decode_jwt_from_request(
-                locations, fresh
-            )
+        jwt_data, jwt_header, jwt_location = _decode_jwt_from_request(
+            locations, fresh, refresh=refresh, verify_type=verify_type
+        )
+
     except NoAuthorizationError:
         if not optional:
             raise
@@ -76,7 +97,7 @@ def verify_jwt_in_request(optional=False, fresh=False, refresh=False, locations=
         _request_ctx_stack.top.jwt_header = {}
         _request_ctx_stack.top.jwt_user = {"loaded_user": None}
         _request_ctx_stack.top.jwt_location = None
-        return
+        return None
 
     # Save these at the very end so that they are only saved in the requet
     # context if the token is valid and all callbacks succeed
@@ -88,7 +109,13 @@ def verify_jwt_in_request(optional=False, fresh=False, refresh=False, locations=
     return jwt_header, jwt_data
 
 
-def jwt_required(optional=False, fresh=False, refresh=False, locations=None):
+def jwt_required(
+    optional: bool = False,
+    fresh: bool = False,
+    refresh: bool = False,
+    locations: LocationType = None,
+    verify_type: bool = True,
+) -> Any:
     """
     A decorator to protect a Flask endpoint with JSON Web Tokens.
 
@@ -113,27 +140,25 @@ def jwt_required(optional=False, fresh=False, refresh=False, locations=None):
         example ``'headers'`` or ``['headers', 'cookies']``. Defaults to ``None``
         which indicates that JWTs will be looked for in the locations defined by the
         ``JWT_TOKEN_LOCATION`` configuration option.
+
+    :param verify_type:
+        If ``True``, the token type (access or refresh) will be checked according
+        to the ``refresh`` argument. If ``False``, type will not be checked and both
+        access and refresh tokens will be accepted.
     """
 
     def wrapper(fn):
         @wraps(fn)
         def decorator(*args, **kwargs):
-            verify_jwt_in_request(optional, fresh, refresh, locations)
-
-            # Compatibility with flask < 2.0
-            if hasattr(current_app, "ensure_sync") and callable(
-                getattr(current_app, "ensure_sync", None)
-            ):
-                return current_app.ensure_sync(fn)(*args, **kwargs)
-
-            return fn(*args, **kwargs)  # pragma: no cover
+            verify_jwt_in_request(optional, fresh, refresh, locations, verify_type)
+            return current_app.ensure_sync(fn)(*args, **kwargs)
 
         return decorator
 
     return wrapper
 
 
-def _load_user(jwt_header, jwt_data):
+def _load_user(jwt_header: dict, jwt_data: dict) -> Optional[dict]:
     if not has_user_lookup():
         return None
 
@@ -145,7 +170,7 @@ def _load_user(jwt_header, jwt_data):
     return {"loaded_user": user}
 
 
-def _decode_jwt_from_headers():
+def _decode_jwt_from_headers() -> Tuple[str, None]:
     header_name = config.header_name
     header_type = config.header_type
 
@@ -189,7 +214,7 @@ def _decode_jwt_from_headers():
     return encoded_token, None
 
 
-def _decode_jwt_from_cookies(refresh):
+def _decode_jwt_from_cookies(refresh: bool) -> Tuple[str, Optional[str]]:
     if refresh:
         cookie_key = config.refresh_cookie_name
         csrf_header_key = config.refresh_csrf_header_name
@@ -215,7 +240,7 @@ def _decode_jwt_from_cookies(refresh):
     return encoded_token, csrf_value
 
 
-def _decode_jwt_from_query_string():
+def _decode_jwt_from_query_string() -> Tuple[str, None]:
     param_name = config.query_string_name
     prefix = config.query_string_value_prefix
 
@@ -233,9 +258,8 @@ def _decode_jwt_from_query_string():
     return encoded_token, None
 
 
-def _decode_jwt_from_json(refresh):
-    content_type = request.content_type or ""
-    if not content_type.startswith("application/json"):
+def _decode_jwt_from_json(refresh: bool) -> Tuple[str, None]:
+    if not request.is_json:
         raise NoAuthorizationError("Invalid content-type. Must be application/json.")
 
     if refresh:
@@ -244,7 +268,7 @@ def _decode_jwt_from_json(refresh):
         token_key = config.json_key
 
     try:
-        encoded_token = request.json.get(token_key, None)
+        encoded_token = request.json and request.json.get(token_key, None)
         if not encoded_token:
             raise BadRequest()
     except BadRequest:
@@ -255,7 +279,12 @@ def _decode_jwt_from_json(refresh):
     return encoded_token, None
 
 
-def _decode_jwt_from_request(locations, fresh, refresh=False):
+def _decode_jwt_from_request(
+    locations: LocationType,
+    fresh: bool,
+    refresh: bool = False,
+    verify_type: bool = True,
+) -> Tuple[dict, dict, str]:
     # Figure out what locations to look for the JWT in this request
     if isinstance(locations, str):
         locations = [locations]
@@ -288,8 +317,6 @@ def _decode_jwt_from_request(locations, fresh, refresh=False):
     # in one place to be valid (not every location).
     errors = []
     decoded_token = None
-    jwt_header = None
-    jwt_location = None
     for location, get_encoded_token_function in get_encoded_token_functions:
         try:
             encoded_token, csrf_token = get_encoded_token_function()
@@ -314,7 +341,9 @@ def _decode_jwt_from_request(locations, fresh, refresh=False):
             raise NoAuthorizationError(errors[0])
 
     # Additional verifications provided by this extension
-    verify_token_type(decoded_token, refresh)
+    if verify_type:
+        verify_token_type(decoded_token, refresh)
+
     if fresh:
         _verify_token_is_fresh(jwt_header, decoded_token)
     verify_token_not_blocklisted(jwt_header, decoded_token)
